package com.ruoyi.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.system.domain.OpenAccount;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.domain.vo.OpenAccountVO;
import com.ruoyi.system.mapper.IUserinfoMapper;
import com.ruoyi.system.mapper.SysUserRoleMapper;
import com.ruoyi.system.service.IUserinfoService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@RequiredArgsConstructor
@Transactional
public class IUserinfoServiceImpl implements IUserinfoService {
    @Autowired
    IUserinfoMapper iUserinfoMapper;
    private final SysUserRoleMapper userRoleMapper;


    @Override
    public R createAdmin(OpenAccountVO openAccountVO) {
        //检查是否存在uId用户
        OpenAccount openAccount1 = checkUIdAndUserName(openAccountVO);
        if (openAccount1 == null){
            //注册用户
            OpenAccount openAccount = new OpenAccount(openAccountVO);
            iUserinfoMapper.insert(openAccount);
            //设置角色
            for (Integer role : openAccountVO.getRoles()) {
                SysUserRole sysUserRole = new SysUserRole();
                sysUserRole.setUserId(openAccount.getUserId());
                sysUserRole.setRoleId(Long.valueOf(role));
                userRoleMapper.insert(sysUserRole);
            }
            return R.ok();
        }else {
            //启用用户
            UpdateWrapper<OpenAccount> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("u_id",openAccountVO.getUId());
            updateWrapper.set("del_flag",0);
            iUserinfoMapper.update(null,updateWrapper);

            //设置角色
            for (Integer role : openAccountVO.getRoles()) {
                SysUserRole sysUserRole = new SysUserRole();
                sysUserRole.setUserId(openAccount1.getUserId());
                sysUserRole.setRoleId(Long.valueOf(role));
                userRoleMapper.insert(sysUserRole);
            }
            return R.ok();
        }
    }

    @Override
    public R createUser(OpenAccountVO openAccountVO) {
        //检查是否存在uId用户
        OpenAccount openAccount1 = checkUIdAndUserName(openAccountVO);
        if (openAccount1 == null){
            //注册用户
            OpenAccount openAccount = new OpenAccount(openAccountVO);
            iUserinfoMapper.insert(openAccount);
            //设置角色
            for (Integer role : openAccountVO.getRoles()) {
                SysUserRole sysUserRole = new SysUserRole();
                sysUserRole.setUserId(openAccount.getUserId());
                sysUserRole.setRoleId(Long.valueOf(role));
                userRoleMapper.insert(sysUserRole);
            }
            return R.ok();
        }else {
            //启用用户
            UpdateWrapper<OpenAccount> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("u_id",openAccountVO.getUId());
            updateWrapper.set("del_flag",0);
            iUserinfoMapper.update(null,updateWrapper);

            //设置角色
            for (Integer role : openAccountVO.getRoles()) {
                SysUserRole sysUserRole = new SysUserRole();
                sysUserRole.setUserId(openAccount1.getUserId());
                sysUserRole.setRoleId(Long.valueOf(role));
                userRoleMapper.insert(sysUserRole);
            }
            return R.ok();
        }
    }

    @Override
    public void disableUser(OpenAccountVO openAccountVO) {
        //检查是否存在uId用户
        OpenAccount openAccount1 = checkUIdAndUserName(openAccountVO);
        if (openAccount1 != null){
            //禁用用户
            UpdateWrapper<OpenAccount> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("u_id",openAccountVO.getUId());
            updateWrapper.set("del_flag",2);
            iUserinfoMapper.update(null,updateWrapper);

            //清除分配角色
            QueryWrapper<SysUserRole> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("user_id",openAccount1.getUserId());
            userRoleMapper.delete(queryWrapper);
        }
    }

    /**
     * 判断用户是否存在
     * */
    public OpenAccount checkUIdAndUserName(OpenAccountVO openAccountVO){
        //判断u_id 和 用户名称是否存在
        QueryWrapper<OpenAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("u_id",openAccountVO.getUId());
        OpenAccount openAccount = iUserinfoMapper.selectOne(queryWrapper);
        return openAccount;
    }
}
